searchData={"content_type":"text/markdown","items":[{"doc":"Saxy is an XML SAX parser and encoder.\n\nSaxy provides functions to parse XML file in both binary and streaming way in compliant\nwith [Extensible Markup Language (XML) 1.0 (Fifth Edition)](https://www.w3.org/TR/xml/).\n\nSaxy also offers DSL and API to build, compose and encode XML document.\nSee \"Encoder\" section below for more information.","ref":"Saxy.html","title":"Saxy","type":"module"},{"doc":"Saxy parser supports two modes of parsing: SAX and simple form.","ref":"Saxy.html#module-parser","title":"Parser - Saxy","type":"module"},{"doc":"SAX is an event driven algorithm for parsing XML documents. A SAX parser takes XML document as the input\nand emits events out to a pre-configured event handler during parsing.\n\nThere are several types of SAX events supported by Saxy:\n\n* `:start_document` - after prolog is parsed.\n* `:start_element` - when open tag is parsed.\n* `:characters` - when a chunk of `CharData` is parsed.\n* `:cdata` - when a chunk of `CData` is parsed.\n* `:end_element` - when end tag is parsed.\n* `:end_document` - when the root element is closed.\n\nSee `Saxy.Handler` for more information.","ref":"Saxy.html#module-sax-mode-simple-api-for-xml","title":"SAX mode (Simple API for XML) - Saxy","type":"module"},{"doc":"Saxy supports parsing XML documents into a simple format. See `Saxy.SimpleForm` for more details.","ref":"Saxy.html#module-simple-form-mode","title":"Simple form mode - Saxy","type":"module"},{"doc":"Saxy **only** supports UTF-8 encoding. It also respects the encoding set in XML document prolog, which means\nthat if the declared encoding is not UTF-8, the parser stops. Anyway, when there is no encoding declared,\nSaxy defaults the encoding to UTF-8.","ref":"Saxy.html#module-encoding","title":"Encoding - Saxy","type":"module"},{"doc":"Saxy supports expanding character references and XML 1.0 predefined entity references, for example `&#65;`\nis expanded to `\"A\"`, `&#x26;` to `\"&\"`, and `&amp;` to `\"&\"`.\n\nSaxy does not expand external entity references, but provides an option to specify how they should be handled.\nSee more in \"Shared options\" section.","ref":"Saxy.html#module-reference-expansion","title":"Reference expansion - Saxy","type":"module"},{"doc":"Saxy does not create atoms during the parsing process.","ref":"Saxy.html#module-creation-of-atoms","title":"Creation of atoms - Saxy","type":"module"},{"doc":"Saxy does not support parsing DTD (Doctype Definition) and XSD schemas. When encountering DTD, the parser simply\nskips that.","ref":"Saxy.html#module-dtd-and-xsd","title":"DTD and XSD - Saxy","type":"module"},{"doc":"* `:expand_entity` - specifies how external entity references should be handled. Three supported strategies respectively are:\n  * `:keep` - keep the original binary, for example `Orange &reg;` will be expanded to `\"Orange &reg;\"`, this is the default strategy.\n  * `:skip` - skip the original binary, for example `Orange &reg;` will be expanded to `\"Orange \"`.\n  * `{mod, fun, args}` - take the applied result of the specified MFA.\n  * `:never` - keep the original binary, including predefined entity reference, e.g. `\"Orange &amp;\"` will remain `\"Orange &amp;\"`\n* `:cdata_as_characters` - `true` to emit CData events as `:characters`. Defaults to `true`.","ref":"Saxy.html#module-shared-options","title":"Shared options - Saxy","type":"module"},{"doc":"Saxy offers two APIs to build simple form and encode XML document.\n\nUse `Saxy.XML` to build and compose XML simple form, then `Saxy.encode!/2`\nto encode the built element into XML binary.\n\n    iex> import Saxy.XML\n    iex> element = element(\"person\", [gender: \"female\"], \"Alice\")\n    {\"person\", [{\"gender\", \"female\"}], [\"Alice\"]}\n    iex> Saxy.encode!(element, [version: \"1.0\"])\n    \"<?xml version=\\\"1.0\\\"?> Alice \"\n\nSee `Saxy.XML` for more XML building APIs.\n\nSaxy also provides `Saxy.Builder` protocol to help composing structs into simple form.\n\n    defmodule Person do\n      @derive {\n        Saxy.Builder,\n        name: \"person\", attributes: [:gender], children: [:name, emails: &__MODULE__.build_emails/1]\n      }\n\n      import Saxy.XML\n\n      defstruct [:name, :gender, emails: []]\n\n      def build_emails(emails) do\n        count = Enum.count(emails)\n\n        element(\n          \"emails\",\n          [count: Enum.count(emails)],\n          Enum.map(emails, &element(\"email\", [], &1))\n        )\n      end\n    end\n\n    iex> jack = %Person{gender: :male, name: \"Jack\", emails: [\"john@example.com\"]}\n    iex> john = %Person{gender: :male, name: \"John\"}\n    iex> import Saxy.XML\n    iex> root = element(\"people\", [], [jack, john])\n    iex> Saxy.encode!(root, [version: \"1.0\"])\n    \"<?xml version=\\\"1.0\\\"?>  Jack  john@example.com    John   \"","ref":"Saxy.html#module-encoder","title":"Encoder - Saxy","type":"module"},{"doc":"Encodes a simple form XML element into string.\n\nThis function encodes an element in simple form format and a prolog to an XML document.","ref":"Saxy.html#encode!/2","title":"Saxy.encode!/2","type":"function"},{"doc":"iex> import Saxy.XML\n    iex> root = element(:foo, [{\"foo\", \"bar\"}], \"bar\")\n    iex> prolog = [version: \"1.0\"]\n    iex> Saxy.encode!(root, prolog)\n    \"<?xml version=\\\"1.0\\\"?> bar \"\n    iex> prolog = [version: \"1.0\", encoding: \"UTF-8\"]\n    iex> Saxy.encode!(root, prolog)\n    \"<?xml version=\\\"1.0\\\" encoding=\\\"UTF-8\\\"?> bar \"","ref":"Saxy.html#encode!/2-examples","title":"Examples - Saxy.encode!/2","type":"function"},{"doc":"Encodes a simple form element into IO data.\n\nSame as `encode!/2` but this encodes the document into IO data.","ref":"Saxy.html#encode_to_iodata!/2","title":"Saxy.encode_to_iodata!/2","type":"function"},{"doc":"iex> import Saxy.XML\n    iex> root = element(:foo, [{\"foo\", \"bar\"}], \"bar\")\n    iex> prolog = [version: \"1.0\"]\n    iex> Saxy.encode_to_iodata!(root, prolog)\n    [\n      [~c'<?xml', [32, ~c'version', 61, 34, \"1.0\", 34], [], [], ~c'?>'],\n      [60, \"foo\", 32, \"foo\", 61, 34, \"bar\", 34],\n      62,\n      [\"bar\"],\n      [60, 47, \"foo\", 62]\n    ]","ref":"Saxy.html#encode_to_iodata!/2-examples","title":"Examples - Saxy.encode_to_iodata!/2","type":"function"},{"doc":"Parses XML stream data.\n\nThis function takes a stream, SAX event handler (see more at `Saxy.Handler`) and an initial state as the input, it returns\n`{:ok, state}` if parsing is successful, otherwise `{:error, exception}`, where `exception` is a\n`Saxy.ParseError` struct which can be converted into readable message with `Exception.message/1`.","ref":"Saxy.html#parse_stream/4","title":"Saxy.parse_stream/4","type":"function"},{"doc":"defmodule MyTestHandler do\n      @behaviour Saxy.Handler\n\n      def handle_event(:start_document, prolog, state) do\n        {:ok, [{:start_document, prolog} | state]}\n      end\n\n      def handle_event(:end_document, _data, state) do\n        {:ok, [{:end_document} | state]}\n      end\n\n      def handle_event(:start_element, {name, attributes}, state) do\n        {:ok, [{:start_element, name, attributes} | state]}\n      end\n\n      def handle_event(:end_element, name, state) do\n        {:ok, [{:end_element, name} | state]}\n      end\n\n      def handle_event(:characters, chars, state) do\n        {:ok, [{:chacters, chars} | state]}\n      end\n    end\n\n    iex> stream = File.stream!(\"./test/support/fixture/foo.xml\")\n    iex> Saxy.parse_stream(stream, MyTestHandler, [])\n    {:ok,\n     [{:end_document},\n      {:end_element, \"foo\"},\n      {:start_element, \"foo\", [{\"bar\", \"value\"}]},\n      {:start_document, [version: \"1.0\"]}]}","ref":"Saxy.html#parse_stream/4-examples","title":"Examples - Saxy.parse_stream/4","type":"function"},{"doc":"`Saxy.parse_stream/3` takes a `File.Stream` or `Stream` as the input, so the amount of bytes to buffer in each\nchunk can be controlled by `File.stream!/3` API.\n\nDuring parsing, the actual memory used by Saxy might be higher than the number configured for each chunk, since\nSaxy holds in memory some parsed parts of the original binary to leverage Erlang sub-binary extracting. Anyway,\nSaxy tries to free those up when it makes sense.","ref":"Saxy.html#parse_stream/4-memory-usage","title":"Memory usage - Saxy.parse_stream/4","type":"function"},{"doc":"See the “Shared options” section at the module documentation.\n\n* `:character_data_max_length` - tells the parser to emit the `:characters` event when its length exceeds the specified\n  number. The option is useful when the tag being parsed containing a very large chunk of data. Defaults to `:infinity`.","ref":"Saxy.html#parse_stream/4-options","title":"Options - Saxy.parse_stream/4","type":"function"},{"doc":"Parses XML binary data.\n\nThis function takes XML binary, SAX event handler (see more at `Saxy.Handler`) and an initial state as the input, it returns\n`{:ok, state}` if parsing is successful, otherwise `{:error, exception}`, where `exception` is a\n`Saxy.ParseError` struct which can be converted into readable message with `Exception.message/1`.\n\nThe third argument `state` can be used to keep track of data and parsing progress when parsing is happening, which will be\nreturned when parsing finishes.","ref":"Saxy.html#parse_string/4","title":"Saxy.parse_string/4","type":"function"},{"doc":"See the “Shared options” section at the module documentation.","ref":"Saxy.html#parse_string/4-options","title":"Options - Saxy.parse_string/4","type":"function"},{"doc":"defmodule MyTestHandler do\n      @behaviour Saxy.Handler\n\n      def handle_event(:start_document, prolog, state) do\n        {:ok, [{:start_document, prolog} | state]}\n      end\n\n      def handle_event(:end_document, _data, state) do\n        {:ok, [{:end_document} | state]}\n      end\n\n      def handle_event(:start_element, {name, attributes}, state) do\n        {:ok, [{:start_element, name, attributes} | state]}\n      end\n\n      def handle_event(:end_element, name, state) do\n        {:ok, [{:end_element, name} | state]}\n      end\n\n      def handle_event(:characters, chars, state) do\n        {:ok, [{:chacters, chars} | state]}\n      end\n    end\n\n    iex> xml = \"<?xml version='1.0' ?>  \"\n    iex> Saxy.parse_string(xml, MyTestHandler, [])\n    {:ok,\n     [{:end_document},\n      {:end_element, \"foo\"},\n      {:start_element, \"foo\", [{\"bar\", \"value\"}]},\n      {:start_document, [version: \"1.0\"]}]}","ref":"Saxy.html#parse_string/4-examples","title":"Examples - Saxy.parse_string/4","type":"function"},{"doc":"Parses XML stream and returns a stream of elements.\n\nThis function takes a stream and returns a stream of xml SAX events.\nWhen any parsing error occurs, it raises a `Saxy.ParseError` exception.","ref":"Saxy.html#stream_events/2","title":"Saxy.stream_events/2","type":"function"},{"doc":"iex> stream = File.stream!(\"./test/support/fixture/foo.xml\")\n    iex> Enum.to_list Saxy.stream_events stream\n    [\n      start_document: [version: \"1.0\"],\n      start_element: {\"foo\", [{\"bar\", \"value\"}]},\n      end_element: \"foo\"\n    ]\n    iex> Enum.to_list Saxy.stream_events [\" unclosed value\"]\n    ** (Saxy.ParseError) unexpected end of input, expected token: :chardata\n\n> #### Warning {: .warning }\n>\n> Input stream is evaluated lazily, therefore some events may be emitted before\n> exception is raised","ref":"Saxy.html#stream_events/2-examples","title":"Examples - Saxy.stream_events/2","type":"function"},{"doc":"`Saxy.stream_events/2` takes a `File.Stream` or `Stream` as the input, so the amount of bytes to buffer in each\nchunk can be controlled by `File.stream!/3` API.\n\nDuring parsing, the actual memory used by Saxy might be higher than the number configured for each chunk, since\nSaxy holds in memory some parsed parts of the original binary to leverage Erlang sub-binary extracting. Anyway,\nSaxy tries to free those up when it makes sense.","ref":"Saxy.html#stream_events/2-memory-usage","title":"Memory usage - Saxy.stream_events/2","type":"function"},{"doc":"See the “Shared options” section at the module documentation.\n\n* `:character_data_max_length` - tells the parser to emit the `:characters` event when its length exceeds the specified\n  number. The option is useful when the tag being parsed containing a very large chunk of data. Defaults to `:infinity`.","ref":"Saxy.html#stream_events/2-options","title":"Options - Saxy.stream_events/2","type":"function"},{"doc":"Protocol for building XML content.","ref":"Saxy.Builder.html","title":"Saxy.Builder","type":"protocol"},{"doc":"This helps to generate XML content simple form in trivial cases.\n\nThere are a few required options:\n\n* `name` - tag name of generated XML element.\n* `attributes` - struct keys to be encoded as attributes.\n* `children` - a list of entries to be collected as element content. Each entry could be either:\n  * key - value will be used as the content.\n  * two-element tuple of key and transformer - struct value will be passed to the transformer.\n    Transformer function could be a captured public function or a tuple of module and function.","ref":"Saxy.Builder.html#module-deriving","title":"Deriving - Saxy.Builder","type":"protocol"},{"doc":"defmodule Person do\n      @derive {\n        Saxy.Builder,\n        name: \"person\", attributes: [:gender], children: [:name, emails: &__MODULE__.build_emails/1]\n      }\n\n      import Saxy.XML\n\n      defstruct [:name, :gender, emails: []]\n\n      def build_emails(emails) do\n        count = Enum.count(emails)\n\n        element(\n          \"emails\",\n          [count: Enum.count(emails)],\n          Enum.map(emails, &element(\"email\", [], &1))\n        )\n      end\n    end\n\n    iex> person = %Person{name: \"Alice\", gender: \"female\", emails: [\"alice@foo.com\", \"alice@bar.com\"]}\n    iex> Saxy.Builder.build(person)\n    {\"person\", [{\"gender\", \"female\"}], [\"Alice\", {\"emails\", [{\"count\", \"2\"}], [{\"email\", [], [\"alice@foo.com\"]}, {\"email\", [], [\"alice@bar.com\"]}]}]}\n\nCustom implementation could be done by implementing protocol:\n\n    defmodule User do\n      defstruct [:username, :name]\n    end\n\n    defimpl Saxy.Builder, for: User do\n      import Saxy.XML\n\n      def build(user) do\n        element(\n          \"Person\",\n          [{\"userName\", user.username}],\n          [element(\"Name\", [], user.name)]\n        )\n      end\n    end\n\n    iex> user = %User{name: \"Alice\", username: \"alice99\"}\n    iex> Saxy.Builder.build(user)\n    {\"Person\", [{\"userName\", \"alice99\"}], [{\"Name\", [], [\"Alice\"]}]}","ref":"Saxy.Builder.html#module-examples","title":"Examples - Saxy.Builder","type":"protocol"},{"doc":"Builds `content` to XML content in simple form.","ref":"Saxy.Builder.html#build/1","title":"Saxy.Builder.build/1","type":"function"},{"doc":"","ref":"Saxy.Builder.html#t:t/0","title":"Saxy.Builder.t/0","type":"type"},{"doc":"This module provides callbacks to implement SAX events handler.","ref":"Saxy.Handler.html","title":"Saxy.Handler","type":"behaviour"},{"doc":"Callback for event handling.\n\nThis callback takes an event type, an event data and `user_state` as the input.\n\nThe initial `user_state` is the third argument in `Saxy.parse_string/3` and `Saxy.parse_stream/3`.\nIt can be accumulated and passed around during the parsing time by returning it as the result of\nthe callback implementation, which can be used to keep track of data when parsing is happening.\n\nReturning `{:ok, new_state}` continues the parsing process with the new state.\n\nReturning `{:stop, anything}` stops the prosing process immediately, and `anything` will be returned.\nThis is usually handy when we want to get the desired return without parsing the whole file.\n\nReturning `{:halt, anything}` stops the prosing process immediately, `anything` will be returned, together\nwith the rest of buffer being parsed. This is usually handy when we want to get the desired return\nwithout parsing the whole file.","ref":"Saxy.Handler.html#c:handle_event/3","title":"Saxy.Handler.handle_event/3","type":"callback"},{"doc":"There are a couple of events that need to be handled in the handler.\n\n* `:start_document`.\n* `:start_element`.\n* `:characters` – the binary that matches [`CharData*`](https://www.w3.org/TR/xml/#d0e1106) and [Reference](https://www.w3.org/TR/xml/#NT-Reference).\n  Note that it is **not trimmed** and includes **ALL** whitespace characters that match `CharData`.\n* `:cdata` – the binary that matches [`CData*`](https://www.w3.org/TR/2006/REC-xml11-20060816/#NT-CData).\n* `:end_document`.\n* `:end_element`.\n\nCheck out `event_data()` type for more information of what are emitted for each event type.","ref":"Saxy.Handler.html#c:handle_event/3-sax-events","title":"SAX Events - Saxy.Handler.handle_event/3","type":"callback"},{"doc":"defmodule MyEventHandler do\n      @behaviour Saxy.Handler\n\n      def handle_event(:start_document, prolog, state) do\n        {:ok, [{:start_document, prolog} | state]}\n      end\n\n      def handle_event(:end_document, _data, state) do\n        {:ok, [{:end_document} | state]}\n      end\n\n      def handle_event(:start_element, {name, attributes}, state) do\n        {:ok, [{:start_element, name, attributes} | state]}\n      end\n\n      def handle_event(:end_element, name, state) do\n        {:ok, [{:end_element, name} | state]}\n      end\n\n      def handle_event(:characters, chars, state) do\n        {:ok, [{:chacters, chars} | state]}\n      end\n    end","ref":"Saxy.Handler.html#c:handle_event/3-examples","title":"Examples - Saxy.Handler.handle_event/3","type":"callback"},{"doc":"","ref":"Saxy.Handler.html#t:cdata_data/0","title":"Saxy.Handler.cdata_data/0","type":"type"},{"doc":"","ref":"Saxy.Handler.html#t:characters_data/0","title":"Saxy.Handler.characters_data/0","type":"type"},{"doc":"","ref":"Saxy.Handler.html#t:end_document_data/0","title":"Saxy.Handler.end_document_data/0","type":"type"},{"doc":"","ref":"Saxy.Handler.html#t:end_element_data/0","title":"Saxy.Handler.end_element_data/0","type":"type"},{"doc":"","ref":"Saxy.Handler.html#t:event_data/0","title":"Saxy.Handler.event_data/0","type":"type"},{"doc":"","ref":"Saxy.Handler.html#t:event_name/0","title":"Saxy.Handler.event_name/0","type":"type"},{"doc":"","ref":"Saxy.Handler.html#t:start_document_data/0","title":"Saxy.Handler.start_document_data/0","type":"type"},{"doc":"","ref":"Saxy.Handler.html#t:start_element_data/0","title":"Saxy.Handler.start_element_data/0","type":"type"},{"doc":"Supports parsing an XML document partially. This module is useful when\nthe XML document cannot be turned into a `Stream` e.g over sockets.","ref":"Saxy.Partial.html","title":"Saxy.Partial","type":"module"},{"doc":"iex> {:ok, partial} = Saxy.Partial.new(StackHandler, [])\n    iex> {:cont, partial} = Saxy.Partial.parse(partial, \" \")\n    iex> {:cont, partial} = Saxy.Partial.parse(partial, \" \")\n    iex> Saxy.Partial.terminate(partial)\n    {:ok,\n     [\n       end_document: {},\n       end_element: \"foo\",\n       start_element: {\"foo\", []},\n       start_document: []\n     ]}","ref":"Saxy.Partial.html#module-example","title":"Example - Saxy.Partial","type":"module"},{"doc":"Obtain the state set by the user.","ref":"Saxy.Partial.html#get_state/1","title":"Saxy.Partial.get_state/1","type":"function"},{"doc":"Builds up a `Saxy.Partial`, which can be used for later parsing.","ref":"Saxy.Partial.html#new/3","title":"Saxy.Partial.new/3","type":"function"},{"doc":"Continue parsing next chunk of the document with a partial.\n\nThis function can return in 3 ways:\n\n* `{:cont, partial}` - The parsing process has not been terminated.\n* `{:halt, user_state}` - The parsing process has been terminated, usually because of parser stopping.\n* `{:halt, user_state, rest}` - The parsing process has been terminated, usually because of parser halting.\n* `{:error, exception}` - The parsing process has erred.","ref":"Saxy.Partial.html#parse/2","title":"Saxy.Partial.parse/2","type":"function"},{"doc":"Same as partial/2, but continue previous parsing with a new, provided state\nas the third argument instead of the previous accumulated state.\n\ni.e.\n`Saxy.Partial.parse(partial, binary, new_state) # coninue previous partial with a new state`\n\nThis function can return in 3 ways:\n\n* `{:cont, partial}` - The parsing process has not been terminated.\n* `{:halt, user_state}` - The parsing process has been terminated, usually because of parser stopping.\n* `{:halt, user_state, rest}` - The parsing process has been terminated, usually because of parser halting.\n* `{:error, exception}` - The parsing process has erred.","ref":"Saxy.Partial.html#parse/3","title":"Saxy.Partial.parse/3","type":"function"},{"doc":"Terminates the XML document parsing.","ref":"Saxy.Partial.html#terminate/1","title":"Saxy.Partial.terminate/1","type":"function"},{"doc":"","ref":"Saxy.Partial.html#t:t/0","title":"Saxy.Partial.t/0","type":"opaque"},{"doc":"","ref":"Saxy.Prolog.html","title":"Saxy.Prolog","type":"module"},{"doc":"","ref":"Saxy.Prolog.html#from_keyword/1","title":"Saxy.Prolog.from_keyword/1","type":"function"},{"doc":"","ref":"Saxy.Prolog.html#t:t/0","title":"Saxy.Prolog.t/0","type":"type"},{"doc":"Provides functions to parse a XML document to\n[simple-form](http://erlang.org/doc/man/xmerl.html#export_simple-3) data structure.","ref":"Saxy.SimpleForm.html","title":"Saxy.SimpleForm","type":"module"},{"doc":"Simple form is a basic representation of the parsed XML document. It contains a root\nelement, and all elements are in the following format:\n\n```\nelement = {tag_name, attributes, content}\ncontent = (element | binary | cdata)*\n```\n\nSee \"Types\" section for more information.","ref":"Saxy.SimpleForm.html#module-data-structure","title":"Data structure - Saxy.SimpleForm","type":"module"},{"doc":"Parse given string into simple form.","ref":"Saxy.SimpleForm.html#parse_string/2","title":"Saxy.SimpleForm.parse_string/2","type":"function"},{"doc":"* `:expand_entity` - specifies how external entity references should be handled. Three supported strategies respectively are:\n  * `:keep` - keep the original binary, for example `Orange &reg;` will be expanded to `\"Orange &reg;\"`, this is the default strategy.\n  * `:skip` - skip the original binary, for example `Orange &reg;` will be expanded to `\"Orange \"`.\n  * `{mod, fun, args}` - take the applied result of the specified MFA.\n  * `:never` - keep the original binary, including predefined entity reference, e.g. `\"Orange &amp;\"` will remain `\"Orange &amp;\"`\n* `:cdata_as_characters` - `true` to return CData as characters, `false` to wrap CData as `{:cdata, data}`. Defaults to `true`.\n\nNote that it is recommended to disable `:cdata_as_characters` if the outcome simple form data is meant to be re-encoded later.\nConsider the following example, the encoded document has different sematics from the original one.\n\n    iex> xml = \" <![CDATA[ Hello, world! ]]> \"\n    iex> {:ok, simple_form} = Saxy.SimpleForm.parse_string(xml, cdata_as_characters: true)\n    {:ok, {\"foo\", [], [\" Hello, world! \"]}}\n    iex> Saxy.encode!(simple_form)\n    \"  Hello, world!  \"","ref":"Saxy.SimpleForm.html#parse_string/2-options","title":"Options - Saxy.SimpleForm.parse_string/2","type":"function"},{"doc":"Given this XML document.\n\n    iex> xml = \"\"\"\n    ...> <?xml version=\"1.0\" encoding=\"utf-8\" ?>\n    ...>  \n    ...>    \n    ...>      Titanic \n    ...>      Jack &amp; Rose \n    ...>    \n    ...>    \n    ...>      Forest Gump \n    ...>      Forest &amp; Jenny \n    ...>    \n    ...>  \n    ...> \"\"\"\n    iex> Saxy.SimpleForm.parse_string(xml)\n    {:ok,\n     {\"menu\", [],\n      [\n        \"\\n  \",\n        {\n          \"movie\",\n          [\n            {\"url\", \"https://www.imdb.com/title/tt0120338/\"},\n            {\"id\", \"tt0120338\"}\n          ],\n          [\n            \"\\n    \",\n            {\"name\", [], [\"Titanic\"]},\n            \"\\n    \",\n            {\"characters\", [], [\"Jack & Rose\"]},\n            \"\\n  \"]\n        },\n        \"\\n  \",\n        {\n          \"movie\",\n          [\n            {\"url\", \"https://www.imdb.com/title/tt0109830/\"},\n            {\"id\", \"tt0109830\"}\n          ],\n          [\n            \"\\n    \",\n            {\"name\", [], [\"Forest Gump\"]},\n            \"\\n    \",\n            {\"characters\", [], [\"Forest & Jenny\"]},\n            \"\\n  \"\n          ]\n        },\n        \"\\n\"\n      ]}}","ref":"Saxy.SimpleForm.html#parse_string/2-examples","title":"Examples - Saxy.SimpleForm.parse_string/2","type":"function"},{"doc":"","ref":"Saxy.SimpleForm.html#t:attributes/0","title":"Saxy.SimpleForm.attributes/0","type":"type"},{"doc":"","ref":"Saxy.SimpleForm.html#t:content/0","title":"Saxy.SimpleForm.content/0","type":"type"},{"doc":"","ref":"Saxy.SimpleForm.html#t:t/0","title":"Saxy.SimpleForm.t/0","type":"type"},{"doc":"","ref":"Saxy.SimpleForm.html#t:tag_name/0","title":"Saxy.SimpleForm.tag_name/0","type":"type"},{"doc":"Helper functions for building XML elements.","ref":"Saxy.XML.html","title":"Saxy.XML","type":"module"},{"doc":"Builds CDATA in simple form.","ref":"Saxy.XML.html#cdata/1","title":"Saxy.XML.cdata/1","type":"function"},{"doc":"Builds characters in simple form.","ref":"Saxy.XML.html#characters/1","title":"Saxy.XML.characters/1","type":"function"},{"doc":"Builds comment in simple form.","ref":"Saxy.XML.html#comment/1","title":"Saxy.XML.comment/1","type":"function"},{"doc":"Builds element in simple form.","ref":"Saxy.XML.html#element/3","title":"Saxy.XML.element/3","type":"function"},{"doc":"Builds empty element in simple form.","ref":"Saxy.XML.html#empty_element/2","title":"Saxy.XML.empty_element/2","type":"function"},{"doc":"Builds processing instruction in simple form.","ref":"Saxy.XML.html#processing_instruction/2","title":"Saxy.XML.processing_instruction/2","type":"function"},{"doc":"Builds reference in simple form.","ref":"Saxy.XML.html#reference/2","title":"Saxy.XML.reference/2","type":"function"},{"doc":"","ref":"Saxy.XML.html#t:cdata/0","title":"Saxy.XML.cdata/0","type":"type"},{"doc":"","ref":"Saxy.XML.html#t:characters/0","title":"Saxy.XML.characters/0","type":"type"},{"doc":"","ref":"Saxy.XML.html#t:comment/0","title":"Saxy.XML.comment/0","type":"type"},{"doc":"","ref":"Saxy.XML.html#t:content/0","title":"Saxy.XML.content/0","type":"type"},{"doc":"","ref":"Saxy.XML.html#t:dec_ref/0","title":"Saxy.XML.dec_ref/0","type":"type"},{"doc":"","ref":"Saxy.XML.html#t:element/0","title":"Saxy.XML.element/0","type":"type"},{"doc":"","ref":"Saxy.XML.html#t:entity_ref/0","title":"Saxy.XML.entity_ref/0","type":"type"},{"doc":"","ref":"Saxy.XML.html#t:hex_ref/0","title":"Saxy.XML.hex_ref/0","type":"type"},{"doc":"","ref":"Saxy.XML.html#t:processing_instruction/0","title":"Saxy.XML.processing_instruction/0","type":"type"},{"doc":"","ref":"Saxy.XML.html#t:ref/0","title":"Saxy.XML.ref/0","type":"type"},{"doc":"Returned when parser encounters errors during parsing.","ref":"Saxy.ParseError.html","title":"Saxy.ParseError","type":"exception"},{"doc":"","ref":"Saxy.ParseError.html#message/1","title":"Saxy.ParseError.message/1","type":"function"},{"doc":"","ref":"Saxy.ParseError.html#t:reason/0","title":"Saxy.ParseError.reason/0","type":"type"},{"doc":"","ref":"Saxy.ParseError.html#t:t/0","title":"Saxy.ParseError.t/0","type":"type"},{"doc":"# Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"* Require Elixir 1.12+.\n* Introduce `Saxy.stream_events` (#118)\n* Several bug fixes and compilation warnings","ref":"changelog.html#v1-6-0-2024-10-22","title":"v1.6.0 - 2024-10-22 - Changelog","type":"extras"},{"doc":"* Saxy.encode! now supports \"UTF-8\"/\"utf-8\" as string in the prolog [#102](https://github.com/qcam/saxy/pull/102).\n* Access user state inside partial to allow streaming output [#100](https://github.com/qcam/saxy/pull/100).\n* Introduce :never to expand_entity option [#114](https://github.com/qcam/saxy/pull/114).\n* Bug fixes around typespecs.","ref":"changelog.html#v1-5-0-2022-11-09","title":"v1.5.0 - 2022-11-09 - Changelog","type":"extras"},{"doc":"* Optimize `Saxy.parse_string` (about 20% faster) [#84](https://github.com/qcam/saxy/pull/84).\n  Previously this could be achieved with application config, you no longer have\n  to do that.\n* Support custom transformer in `Saxy.Builder` [#87](https://github.com/qcam/saxy/pull/87).\n* Miscellaneous fixes [#81](https://github.com/qcam/saxy/pull/81),\n  [#82](https://github.com/qcam/saxy/pull/82),\n  [#89](https://github.com/qcam/saxy/pull/89).","ref":"changelog.html#v1-4-0-2021-06-05","title":"v1.4.0 - 2021-06-05 - Changelog","type":"extras"},{"doc":"* Fix Saxy.Builder primitive type implementations.\n* Add `:cdata` SAX event type.\n* Introduce `:cdata_as_characters` option in parsers.","ref":"changelog.html#v1-3-0-2020-10-18","title":"v1.3.0 - 2020-10-18 - Changelog","type":"extras"},{"doc":"* Bring back accidentally removed XML builders.","ref":"changelog.html#v1-2-2-2020-10-02","title":"v1.2.2 - 2020-10-02 - Changelog","type":"extras"},{"doc":"* Fix incorrect type spec.","ref":"changelog.html#v1-2-1-2020-09-27-retired","title":"v1.2.1 - 2020-09-27 (retired) - Changelog","type":"extras"},{"doc":"* Fix XML encoding without prolog [#57](https://github.com/qcam/saxy/pull/57).\n* Fix integer typespec [#58](https://github.com/qcam/saxy/pull/58).\n* Introduce parser halting [#66](https://github.com/qcam/saxy/pull/66).\n* Speed up XML builder [#69](https://github.com/qcam/saxy/pull/69).","ref":"changelog.html#v1-2-0-2020-06-02","title":"v1.2.0 - 2020-06-02 - Changelog","type":"extras"},{"doc":"* Introduce `:character_data_max_length` option in stream and partial parsing.","ref":"changelog.html#v1-1-0-2020-02-09","title":"v1.1.0 - 2020-02-09 - Changelog","type":"extras"},{"doc":"* Support Elixir 1.6+.\n* Fix white spaces emitting bug in empty elements.\n* **BREAKING CHANGES:**\n  * Leading whitespaces before a tag open will be emitted.","ref":"changelog.html#v1-0-0-2019-12-19","title":"v1.0.0 - 2019-12-19 - Changelog","type":"extras"},{"doc":"* Add support of partial parsing [#42](https://github.com/qcam/saxy/pull/42).","ref":"changelog.html#v0-10-0-2019-08-26","title":"v0.10.0 - 2019-08-26 - Changelog","type":"extras"},{"doc":"* Fix type spec warnings in `Saxy.SimpleForm`.","ref":"changelog.html#v0-9-1-2019-02-26","title":"v0.9.1 - 2019-02-26 - Changelog","type":"extras"},{"doc":"* Allow turning off streaming feature in config [#30](https://github.com/qcam/saxy/pull/30).\n* Skip DTD instead of crashing [#33](https://github.com/qcam/saxy/pull/33).\n* Minor bug fix on element attributes order [09b90a9b50ea3ffa17ba2736c29ff791ff9859d0](https://github.com/qcam/saxy/commit/09b90a9b50ea3ffa17ba2736c29ff791ff9859d0).","ref":"changelog.html#v0-9-0-2018-10-21","title":"v0.9.0 - 2018-10-21 - Changelog","type":"extras"},{"doc":"* Improve streaming parsing [#23](https://github.com/qcam/saxy/pull/23).\n* Improve parser performance [#24](https://github.com/qcam/saxy/pull/24).\n* Improve parser error handling [#25](https://github.com/qcam/saxy/pull/25).","ref":"changelog.html#v0-8-0-2018-09-05","title":"v0.8.0 - 2018-09-05 - Changelog","type":"extras"},{"doc":"* Introduce XML encoder [#17](https://github.com/qcam/saxy/pull/17).\n* Fix wrong ASCII code point matching [#20](https://github.com/qcam/saxy/pull/20).\n* Brought back UTF-8 encoding validation ([#16](https://github.com/qcam/saxy/pull/16)).","ref":"changelog.html#v0-7-0-2018-07-14","title":"v0.7.0 - 2018-07-14 - Changelog","type":"extras"},{"doc":"* Introduce `:expand_entity` option ([#14](https://github.com/qcam/saxy/pull/14)).\n* Hard deprecate `Saxy.Handler.handle_entity_reference/1` callback ([#14](https://github.com/qcam/saxy/pull/14)).\n* Fix a UTF-8 buffering bug for streaming parsing ([#13](https://github.com/qcam/saxy/pull/13), [#15](https://github.com/qcam/saxy/pull/15)).\n* Return only root tag in simple form parsing ([e8c062](https://github.com/qcam/saxy/commit/e8c062e94f91ccea4491cec29c4c7861e7b7163b)).","ref":"changelog.html#v0-6-0-2018-04-08","title":"v0.6.0 - 2018-04-08 - Changelog","type":"extras"},{"doc":"* Introduce Simple Form parsing.\n* Parse misc after finishing parsing root element.","ref":"changelog.html#v0-5-0-2018-03-15","title":"v0.5.0 - 2018-03-15 - Changelog","type":"extras"},{"doc":"* Supported Elixir v1.3.\n* Increased parsing speed by 22 times.\n* Improved returning error.\n* Added `handle_entity_reference` callback in `Saxy.Handler`.\n\n**Breaking changes:**\n\n* Required entity reference handling in `Saxy.Handler`.","ref":"changelog.html#v0-4-0-2018-03-12","title":"v0.4.0 - 2018-03-12 - Changelog","type":"extras"},{"doc":"Copyright (c) 2018 Cẩm Huỳnh\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the \"Software\"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"License","type":"extras"},{"doc":"Saxy\n====\n\n[![Test suite](https://github.com/qcam/saxy/actions/workflows/test.yml/badge.svg)](https://github.com/qcam/saxy/actions/workflows/test.yml)\n[![Module Version](https://img.shields.io/hexpm/v/saxy.svg)](https://hex.pm/packages/saxy)\n\nSaxy (Sá xị) is an XML SAX parser and encoder in Elixir that focuses on speed, usability and standard compliance.\n\nComply with [Extensible Markup Language (XML) 1.0 (Fifth Edition)](https://www.w3.org/TR/xml/).","ref":"readme.html","title":"Saxy","type":"extras"},{"doc":"* An incredibly fast XML 1.0 SAX parser.\n* An extremely fast XML encoder.\n* Native support for streaming parsing large XML files.\n* Parse XML documents into simple DOM format.\n* Support quick returning in event handlers.","ref":"readme.html#features-highlight","title":"Features highlight - Saxy","type":"extras"},{"doc":"Add `:saxy` to your `mix.exs`.\n\n```elixir\ndef deps() do\n  [\n    {:saxy, \"~> 1.6\"}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - Saxy","type":"extras"},{"doc":"Full documentation is available on [HexDocs](https://hexdocs.pm/saxy/).\n\nIf you never work with a SAX parser before, please check out [this\nguide][sax-guide].","ref":"readme.html#overview","title":"Overview - Saxy","type":"extras"},{"doc":"A SAX event handler implementation is required before starting parsing.\n\n```elixir\ndefmodule MyEventHandler do\n  @behaviour Saxy.Handler\n\n  def handle_event(:start_document, prolog, state) do\n    IO.inspect(\"Start parsing document\")\n    {:ok, [{:start_document, prolog} | state]}\n  end\n\n  def handle_event(:end_document, _data, state) do\n    IO.inspect(\"Finish parsing document\")\n    {:ok, [{:end_document} | state]}\n  end\n\n  def handle_event(:start_element, {name, attributes}, state) do\n    IO.inspect(\"Start parsing element #{name} with attributes #{inspect(attributes)}\")\n    {:ok, [{:start_element, name, attributes} | state]}\n  end\n\n  def handle_event(:end_element, name, state) do\n    IO.inspect(\"Finish parsing element #{name}\")\n    {:ok, [{:end_element, name} | state]}\n  end\n\n  def handle_event(:characters, chars, state) do\n    IO.inspect(\"Receive characters #{chars}\")\n    {:ok, [{:characters, chars} | state]}\n  end\n\n  def handle_event(:cdata, cdata, state) do\n    IO.inspect(\"Receive CData #{cdata}\")\n    {:ok, [{:cdata, cdata} | state]}\n  end\nend\n```\n\nThen start parsing XML documents with:\n\n```elixir\niex> xml = \"<?xml version='1.0' ?>  \"\niex> Saxy.parse_string(xml, MyEventHandler, [])\n{:ok,\n [{:end_document},\n  {:end_element, \"foo\"},\n  {:start_element, \"foo\", [{\"bar\", \"value\"}]},\n  {:start_document, [version: \"1.0\"]}]}\n```","ref":"readme.html#sax-parser","title":"SAX parser - Saxy","type":"extras"},{"doc":"Saxy also accepts file stream as the input:\n\n```elixir\nstream = File.stream!(\"/path/to/file\")\n\nSaxy.parse_stream(stream, MyEventHandler, initial_state)\n```\n\nIt even supports parsing a normal stream.\n\n```elixir\nstream = File.stream!(\"/path/to/file\") |> Stream.filter(&(&1 != \"\\n\"))\n\nSaxy.parse_stream(stream, MyEventHandler, initial_state)\n```","ref":"readme.html#streaming-parsing","title":"Streaming parsing - Saxy","type":"extras"},{"doc":"Saxy can parse an XML document partially. This feature is useful when the\ndocument cannot be turned into a stream e.g receiving over socket.\n\n```elixir\n{:ok, partial} = Partial.new(MyEventHandler, initial_state)\n{:cont, partial} = Partial.parse(partial, \" \")\n{:cont, partial} = Partial.parse(partial, \"  \")\n{:cont, partial} = Partial.parse(partial, \" \")\n{:ok, state} = Partial.terminate(partial)\n```","ref":"readme.html#partial-parsing","title":"Partial parsing - Saxy","type":"extras"},{"doc":"Sometimes it will be convenient to just export the XML document into simple DOM\nformat, which is a 3-element tuple including the tag name, attributes, and a\nlist of its children.\n\n`Saxy.SimpleForm` module has this nicely supported:\n\n```elixir\nSaxy.SimpleForm.parse_string(data)\n\n{\"menu\", [],\n [\n   {\"movie\",\n    [{\"id\", \"tt0120338\"}, {\"url\", \"https://www.imdb.com/title/tt0120338/\"}],\n    [{\"name\", [], [\"Titanic\"]}, {\"characters\", [], [\"Jack &amp; Rose\"]}]},\n   {\"movie\",\n    [{\"id\", \"tt0109830\"}, {\"url\", \"https://www.imdb.com/title/tt0109830/\"}],\n    [\n      {\"name\", [], [\"Forest Gump\"]},\n      {\"characters\", [], [\"Forest &amp; Jenny\"]}\n    ]}\n ]}\n```","ref":"readme.html#simple-dom-format-exporting","title":"Simple DOM format exporting - Saxy","type":"extras"},{"doc":"Saxy offers two APIs to build simple form and encode XML document.\n\nUse `Saxy.XML` to build and compose XML simple form, then `Saxy.encode!/2`\nto encode the built element into XML binary.\n\n```elixir\niex> import Saxy.XML\niex> element = element(\"person\", [gender: \"female\"], \"Alice\")\n{\"person\", [{\"gender\", \"female\"}], [{:characters, \"Alice\"}]}\niex> Saxy.encode!(element, [])\n\"<?xml version=\\\"1.0\\\"?> Alice \"\n```\n\nSee `Saxy.XML` for more XML building APIs.\n\nSaxy also provides `Saxy.Builder` protocol to help composing structs into simple form.\n\n```elixir\ndefmodule Person do\n  @derive {Saxy.Builder, name: \"person\", attributes: [:gender], children: [:name]}\n\n  defstruct [:gender, :name]\nend\n\niex> jack = %Person{gender: :male, name: \"Jack\"}\niex> john = %Person{gender: :male, name: \"John\"}\niex> import Saxy.XML\niex> root = element(\"people\", [], [jack, john])\niex> Saxy.encode!(root, [])\n\"<?xml version=\\\"1.0\\\"?>  Jack  John  \"\n```","ref":"readme.html#xml-builder","title":"XML builder - Saxy","type":"extras"},{"doc":"","ref":"readme.html#faqs-with-saxy-xmls","title":"FAQs with Saxy/XMLs - Saxy","type":"extras"},{"doc":"Saxy does not have offer XML to maps conversion, because many awesome people\nalready made it happen 💪:\n\n* https://github.com/bennyhat/xml_json\n* https://github.com/xinz/sax_map\n\nAlternatively, this [pull request](https://github.com/qcam/saxy/pull/78) could\nserve as a good reference if you want to implement your own map-based handler.","ref":"readme.html#saxy-sounds-cool-but-i-just-wanted-to-quickly-convert-some-xmls-into-maps-json","title":"Saxy sounds cool! But I just wanted to quickly convert some XMLs into maps/JSON... - Saxy","type":"extras"},{"doc":"Saxy in its core is a SAX parser, therefore Saxy does not, and likely will\nnot, offer any XPath functionality.\n\n[SweetXml][sweet_xml] is a wonderful library to work with XPath. However,\n`:xmerl`, the library used by SweetXml, is not always memory efficient and\nspeedy. You can combine the best of both sides with [Saxmerl][saxmerl], which\nis a Saxy extension converting XML documents into SweetXml compatible format.\nPlease check that library out for more information.","ref":"readme.html#does-saxy-work-with-xpath","title":"Does Saxy work with XPath? - Saxy","type":"extras"},{"doc":"![Sa xi Chuong Duong](./assets/saxi.jpg)\n\nSa Xi, pronounced like `sa-see`, is an awesome soft drink made by [Chuong Duong](http://www.cdbeco.com.vn/en).","ref":"readme.html#saxy-where-did-the-name-come-from","title":"Saxy! Where did the name come from? - Saxy","type":"extras"},{"doc":"Note that benchmarking XML parsers is difficult and highly depends on the complexity\nof the documents being parsed. Event I try hard to make the benchmarking suite\nfair but it's hard to avoid biases when choosing the documents to benchmark\nagainst.\n\nTherefore the conclusion in this section is only for reference purpose. Please\nfeel free to benchmark against your target documents. The benchmark suite can be found\nin [bench/](https://github.com/qcam/saxy/tree/master/bench).\n\nA rule of thumb is that we should compare apple to apple. Some XML parsers\ntarget only specific types of XML. Therefore some indicators are provided in the\ntest suite to let know of the fairness of the benchmark results.\n\nSome quick and biased conclusions from the benchmark suite:\n\n* For SAX parser, Saxy is usually 1.4 times faster than [Erlsom](https://github.com/willemdj/erlsom).\n  With deeply nested documents, Saxy is noticeably faster (4 times faster).\n* For XML builder and encoding, Saxy is usually 10 to 30 times faster than [XML Builder](https://github.com/joshnuss/xml_builder).\n  With deeply nested documents, it could be 180 times faster.\n* Saxy significantly uses less memory than XML Builder (4 times to 25 times).\n* Saxy significantly uses less memory than Xmerl, Erlsom and Exomler (1.4 times\n  10 times).","ref":"readme.html#benchmarking","title":"Benchmarking - Saxy","type":"extras"},{"doc":"* No XSD supported.\n* No DTD supported, when Saxy encounters a `<!DOCTYPE`, it skips that.\n* Only support UTF-8 encoding.","ref":"readme.html#limitations","title":"Limitations - Saxy","type":"extras"},{"doc":"If you have any issues or ideas, feel free to write to https://github.com/qcam/saxy/issues.\n\nTo start developing:\n\n1. Fork the repository.\n2. Write your code and related tests.\n3. Create a pull request at https://github.com/qcam/saxy/pulls.","ref":"readme.html#contributing","title":"Contributing - Saxy","type":"extras"},{"doc":"Copyright (c) 2018 Cẩm Huỳnh\n\nThis software is licensed under [the MIT license](./LICENSE.md).\n\n[saxmerl]: https://github.com/qcam/saxmerl\n[sweet_xml]: https://github.com/kbrw/sweet_xml\n[sax-guide]: https://hexdocs.pm/saxy/getting-started-with-sax.html","ref":"readme.html#copyright-and-license","title":"Copyright and License - Saxy","type":"extras"},{"doc":"# Getting started with SAX\n\nThis guide is an introduction of how you could parse a XML document in SAX mode\nwith Saxy.","ref":"getting-started-with-sax.html","title":"Getting started with SAX","type":"extras"},{"doc":"SAX is an event driven algorithm to parse XML documents, which means that during\nparsing, a SAX parser will emit any meaningful data of the document such as\nstart tag to a pre-configured handler, then the handler decides how to process\nwith the emitted data.\n\nSAX is especially useful when it comes to large file parsing, because unlike DOM\nparsing, it does not require fitting the whole parsed document into memory (for\nXPath operations for example).\n\nParsing in SAX mode is efficient, but it would take some time to get used to.\nThis guide is here to help you get over it.","ref":"getting-started-with-sax.html#sax-simple-api-for-xml","title":"SAX (Simple API for XML) - Getting started with SAX","type":"extras"},{"doc":"Given a XML document as below needs to be parsed, and the desired outcome\nwill be a list of foods with name, price and description.\n\n```XML\n<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n \n   \n     Belgian Waffles \n     $5.95 \n     Two of our famous Belgian Waffles with plenty of real maple syrup \n   \n   \n     Strawberry Belgian Waffles \n     $7.95 \n     Light Belgian waffles covered with strawberries and whipped cream \n   \n \n```\n\nTo parse a XML document the SAX way with Saxy, first you need to implement a\nhandler.\n\nLet's start with handling the start and end events of the document. No action to\ntake here, we simply return whatever passed in.\n\n```\ndefmodule FoodHandler do\n  @behaviour Saxy.Handler\n\n  def handle_event(:start_document, _prolog, state) do\n    {:ok, state}\n  end\n\n  def handle_event(:end_document, _, {_current_tag, foods}) do\n    {:ok, foods}\n  end\nend\n```\n\nNext we will be handling the ` ` element. The action will be very simple as\nwell.\n\n* When ` ` element starts, we put a new struct into the food list.\n* When ` ` element ends, we do nothing but return the list.\n\nTo make it clear, let's call the state `foods` instead of `state`.\n\n    defmodule FoodHandler do\n      @behaviour Saxy.Handler\n\n      def handle_event(:start_element, {name, _attributes}, foods) do\n        if name == \"food\" do\n          {:ok, [%Food{} | foods]}\n        else\n          {:ok, foods}\n        end\n      end\n\n      def handle_event(:end_element, _data, foods) do\n        {:ok, foods}\n      end\n    end\n\nNow we shall start handling ` ` and its content. But we encounter a problem:\n`:characters` event, which we are supposed to get \"Belgian Waffles\" for the\nfirst food name does not include which tag it belongs to.\n\nSo we need to somehow cache the current tag that is being parsed, let's revise\nour handler a little bit.\n\n    def handle_event(:start_element, {tag_name, _attributes}, {current_tag, foods}) do\n      if tag_name == \"food\" do\n        foods = [%Food{} | foods]\n        {:ok, {tag_name, foods}}\n      else\n        {:ok, {tag_name, foods}}\n      end\n    end\n\nWith this now we can import the content of \"name\", and probably other food\nproperties too.\n\n    def handle_event(:characters, content, {current_tag, foods}) do\n    case foods do\n    [] -> {:ok, {current_tag, foods}}\n      \n\n    _ -> \n    [current_food | foods] = foods\n      food =\n        case current_tag do\n          \"name\" ->\n            Map.put(current_food, :name, content)\n\n          \"price\" ->\n            Map.put(current_food, :price, content)\n\n          \"description\" ->\n            Map.put(current_food, :description, content)\n\n          _other ->\n            current_food\n        end\n\n      {:ok, {\"food\", [food | foods]}}\n    end\n    end \n\nAs now we have implemented the event handler, it is time to parse the document.\n\n    document = File.read!(\"/path/to/the/file\")\n    Saxy.parse_string(document, FoodHandler, {nil, []})\n    {:ok,\n     [\n       %Food{name: \"Belgian Waffles\", price: \"$5.95\", description: \"Two of our famous Belgian Waffles with plenty of real maple syrup\"},\n       %Food{name: \"Strawberry Belgian Waffles\", price: \"$7.95\", description: \"Light Belgian waffles covered with strawberries and whipped cream\"},\n     ]}","ref":"getting-started-with-sax.html#implement-the-handler","title":"Implement the handler - Getting started with SAX","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,50,52,46,50]}}